{-|
Module      : Gargantext.Database.Triggers.NodesNodes
Description : Triggers configuration
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

Triggers on NodesNodes table.

-}

{-# LANGUAGE QuasiQuotes #-}

module Gargantext.Database.Admin.Trigger.NodesContexts
  where

-- (ListId, CorpusId, NodeId)
import Database.PostgreSQL.Simple.SqlQQ (sql)
import Gargantext.Core
import Gargantext.Database.Admin.Config ()
import Gargantext.Database.Admin.Types.Node -- (ListId, CorpusId, NodeId)
import Gargantext.Database.Prelude (execPGSQuery, DBCmd)
import Gargantext.Prelude
import qualified Database.PostgreSQL.Simple as DPS

type MasterListId = ListId

triggerInsertCount :: MasterListId -> DBCmd err Int64
triggerInsertCount lId = execPGSQuery query (lId, toDBid NodeList)
  where
    query :: DPS.Query
    query = [sql|
      CREATE OR REPLACE FUNCTION set_insert_count() RETURNS trigger
        AS $$
          BEGIN
                INSERT INTO node_node_ngrams (node1_id, node2_id, ngrams_id, ngrams_type, weight)
                SELECT lists.parent_id
                     , lists.id
                     , cnn.ngrams_id
                     , cnn.ngrams_type
                     , count(*)         AS weight
                FROM NEW as new1
                    INNER JOIN contexts            doc ON doc.id          = new1.context_id
                    INNER JOIN nodes             lists ON lists.parent_id = new1.node_id
                    INNER JOIN context_node_ngrams cnn ON cnn.context_id  = doc.id
                    WHERE lists.id in (?, lists.id)
                      AND lists.typename = ?
                    GROUP BY lists.parent_id, lists.id, cnn.ngrams_id, cnn.ngrams_type
                ON CONFLICT (node1_id, node2_id, ngrams_id, ngrams_type)
                   DO UPDATE set weight = node_node_ngrams.weight + excluded.weight
                    ;
            RETURN NULL;
          END
        $$ LANGUAGE plpgsql;

        DROP   TRIGGER IF EXISTS trigger_insert_count              ON nodes_contexts;
        CREATE TRIGGER           trigger_insert_count AFTER INSERT ON nodes_contexts
        REFERENCING NEW TABLE AS NEW
        FOR EACH STATEMENT
        EXECUTE PROCEDURE set_insert_count();
  |]


triggerUpdateAdd :: MasterListId -> DBCmd err Int64
triggerUpdateAdd lId = execPGSQuery query (lId, toDBid NodeList)
  where
    query :: DPS.Query
    query = [sql|
       CREATE OR REPLACE FUNCTION set_update_ngrams_add() RETURNS trigger AS $$
        BEGIN
               UPDATE node_node_ngrams nnn0 SET weight = weight + d.fix_count
               FROM ( SELECT lists.parent_id AS node1_id
                    , lists.id               AS node2_id
                    , cnn.ngrams_id          AS ngrams_id
                    , cnn.ngrams_type        AS ngrams_type
                    , count(*)               AS fix_count
                       FROM NEW as new1
                       INNER JOIN contexts    doc         ON doc.id          = new1.context_id
                       INNER JOIN nodes       lists       ON lists.parent_id = new1.node_id
                       INNER JOIN context_node_ngrams cnn ON cnn.context_id  = doc.id
                      WHERE lists.id in (?, lists.id)  -- (masterList_id, userLists)
                        AND lists.typename = ?
                      GROUP BY node1_id, node2_id, ngrams_id, ngrams_type
                    ) as d
                WHERE nnn0.node1_id    = d.node1_id
                AND   nnn0.node2_id    = d.node2_id
                AND   nnn0.ngrams_id   = d.ngrams_id
                AND   nnn0.ngrams_type = d.ngrams_type
                ;
          RETURN NULL;
        END
        $$ LANGUAGE plpgsql;

        DROP trigger IF EXISTS trigger_count_update_add on nodes_contexts;
        CREATE TRIGGER trigger_count_update_add AFTER UPDATE on nodes_contexts
        REFERENCING OLD TABLE AS OLD
                    NEW TABLE AS NEW
        FOR EACH ROW
        WHEN (OLD.category <= 0 AND NEW.category >= 1)
        EXECUTE PROCEDURE set_update_ngrams_add();
  |]


triggerUpdateDel :: MasterListId -> DBCmd err Int64
triggerUpdateDel lId = execPGSQuery query (lId, toDBid NodeList)
  where
    query :: DPS.Query
    query = [sql|
       CREATE OR REPLACE FUNCTION set_update_ngrams_count_del() RETURNS trigger AS $$
        BEGIN
               UPDATE node_node_ngrams nnn0 SET weight = weight - d.fix_count
               FROM ( SELECT lists.parent_id AS node1_id
                    , lists.id               AS node2_id
                    , cnn.ngrams_id          AS ngrams_id
                    , cnn.ngrams_type        AS ngrams_type
                    , count(*)               AS fix_count
                         FROM NEW AS new1
                         INNER JOIN contexts    doc         ON doc.id          = new1.context_id
                         INNER JOIN nodes       lists       ON new1.node_id    = lists.parent_id
                         INNER JOIN context_node_ngrams cnn ON cnn.context_id  = doc.id
                        WHERE lists.id in (?, lists.id)  -- (masterList_id, userLists)
                          AND lists.typename = ?
                        GROUP BY node1_id, node2_id, ngrams_id, ngrams_type
                    ) as d
                WHERE nnn0.node1_id    = d.node1_id
                AND   nnn0.node2_id    = d.node2_id
                AND   nnn0.ngrams_id   = d.ngrams_id
                AND   nnn0.ngrams_type = d.ngrams_type
                ;
          RETURN NULL;
        END
        $$ LANGUAGE plpgsql;

        DROP   TRIGGER IF EXISTS trigger_count_delete2              ON nodes_contexts;
        CREATE TRIGGER           trigger_count_delete2 AFTER UPDATE ON nodes_contexts
        REFERENCING OLD TABLE AS OLD
                    NEW TABLE AS NEW
        FOR EACH ROW
        WHEN (OLD.category >= 1 AND NEW.category <= 0)
        EXECUTE PROCEDURE set_update_ngrams_count_del();
  |]
